@@ -1,3 +1,4 @@
 from glotter.settings import Settings
 from glotter.decorators import project_test, project_fixture
-from glotter.__main__ import main as main
\ No newline at end of file
+from glotter.__main__ import main
+from glotter.test_doc_generator import generate_test_docs
