@@ -1,10 +1,12 @@
 import os
+from functools import lru_cache
 
 import yaml
 
 from glotter import testinfo
 from glotter.settings import Settings
 from glotter.containerfactory import ContainerFactory
+from glotter.utils import error_and_exit
 
 
 class Source:
@@ -55,15 +57,17 @@ class Source:
         return self._test_info
 
     def __repr__(self):
-        return f'Source(name: {self.name}, path: {self.path})'
+        return f"Source(name: {self.name}, path: {self.path})"
 
-    def build(self, params=''):
+    def build(self, params=""):
         if self.test_info.container_info.build is not None:
-            command = f'{self.test_info.container_info.build} {params}'
+            command = f"{self.test_info.container_info.build} {params}"
             result = self._container_exec(command)
             if result[0] != 0:
-                raise RuntimeError(f'unable to build using cmd "{self.test_info.container_info.build} {params}":\n'
-                                   f'{result[1].decode("utf-8")}')
+                raise RuntimeError(
+                    f'unable to build using cmd "{self.test_info.container_info.build} {params}":\n'
+                    f'{result[1].decode("utf-8")}'
+                )
 
     def run(self, params=None):
         """
@@ -72,10 +76,10 @@ class Source:
         :param params: input passed to the source as it's run
         :return: the output of running the source
         """
-        params = params or ''
-        command = f'{self.test_info.container_info.cmd} {params}'
+        params = params or ""
+        command = f"{self.test_info.container_info.cmd} {params}"
         result = self._container_exec(command)
-        return result[1].decode('utf-8')
+        return result[1].decode("utf-8")
 
     def exec(self, command):
         """
@@ -85,7 +89,7 @@ class Source:
         :return:  the output of the command as a string
         """
         result = self._container_exec(command)
-        return result[1].decode('utf-8')
+        return result[1].decode("utf-8")
 
     def _container_exec(self, command):
         """
@@ -98,13 +102,14 @@ class Source:
         return container.exec_run(
             cmd=command,
             detach=False,
-            workdir='/src',
+            workdir="/src",
         )
 
     def cleanup(self):
         ContainerFactory().cleanup(self)
 
 
+@lru_cache
 def get_sources(path):
     """
     Walk through a directory and create Source objects
@@ -113,15 +118,77 @@ def get_sources(path):
     :return: a dict where the key is the ProjectType and the value is a list of all the Source objects of that project
     """
     sources = {k: [] for k in Settings().projects}
-    for root, dirs, files in os.walk(path):
+    for root, _, files in os.walk(path):
         path = os.path.abspath(root)
         if "testinfo.yml" in files:
-            with open(os.path.join(path, 'testinfo.yml'), 'r') as file:
+            with open(
+                os.path.join(path, "testinfo.yml"), "r", encoding="utf-8"
+            ) as file:
                 test_info_string = file.read()
-            folder_info = testinfo.FolderInfo.from_dict(yaml.safe_load(test_info_string)['folder'])
-            folder_project_names = folder_info.get_project_mappings(include_extension=True)
+            folder_info = testinfo.FolderInfo.from_dict(
+                yaml.safe_load(test_info_string)["folder"]
+            )
+            folder_project_names = folder_info.get_project_mappings(
+                include_extension=True
+            )
             for project_type, project_name in folder_project_names.items():
                 if project_name in files:
-                    source = Source(project_name, os.path.basename(path), path, test_info_string)
+                    source = Source(
+                        project_name, os.path.basename(path), path, test_info_string
+                    )
                     sources[project_type].append(source)
     return sources
+
+
+def filter_sources(args, sources):
+    """
+    Filter sources based language, project, and/or source
+
+    :param args: Arguments indicating what to filter on
+    :param sources: a dict where the key is the ProjectType and the value is a list of all the Source objects of that project
+    :return: a dict where the key is the ProjectType and the value is a list of all the Source objects of that project
+        that match the filter
+    """
+
+    if args.project:
+        if args.project not in sources:
+            error_and_exit(f'No valid sources found for project: "{args.project}"')
+
+        sources = {args.project: sources[args.project]}
+
+    filtered_sources_by_type = {}
+    for project_type, sources_by_type in sources.items():
+        filtered_sources = [
+            source for source in sources_by_type if _matches_source(args, source)
+        ]
+        if filtered_sources:
+            filtered_sources_by_type[project_type] = filtered_sources
+
+    if not filtered_sources_by_type:
+        errors = []
+        if args.project:
+            errors.append(f'project "{args.project}"')
+
+        if args.language:
+            errors.append(f'language "{args.language}"')
+
+        if args.source:
+            errors.append(f'source "{args.source}"')
+
+        if errors:
+            error_msg = ", ".join(errors)
+            error_and_exit(
+                f"No valid sources found for the following combination: {error_msg}"
+            )
+
+    return filtered_sources_by_type
+
+
+def _matches_source(args, source):
+    if args.language and source.language.lower() != args.language.lower():
+        return False
+
+    return (
+        not args.source
+        or f"{source.name}{source.extension}".lower() == args.source.lower()
+    )
