@@ -5,9 +5,12 @@ import pytest
 
 from glotter.source import get_sources, filter_sources
 from glotter.settings import Settings
+from glotter.test_generator import generate_tests
+from glotter.utils import error_and_exit
 
 
 def test(args):
+    generate_tests()
     test_args = ["-n", "auto"] if args.parallel else []
     if not (args.language or args.project or args.source):
         _run_pytest_and_exit(*test_args)
@@ -19,7 +22,7 @@ def test(args):
             test_args += _get_tests(project_type, all_tests, source)
 
     if not test_args:
-        _error_and_exit("No tests were found")
+        error_and_exit("No tests were found")
 
     _run_pytest_and_exit(*test_args)
 
@@ -45,11 +48,6 @@ def _run_pytest_and_exit(*args):
     sys.exit(code)
 
 
-def _error_and_exit(msg):
-    print(msg)
-    sys.exit(1)
-
-
 class TestCollectionPlugin:
     def __init__(self):
         self.collected = []
